; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; timeout processing
; version 0.2.1 date 2006-06-17
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; reset timeout
; uses: TMP
;  - used stack depth (without call to this function): 0
TIMEOUT_RESET:
        ldi     TMP,50                          ; restart timeout (5.0 s)
        mov     TIMEOUT_TICKS,TMP
        clr     TIMEOUT_FRAME                   ; (re)start timeout movie (after timeout)
        ret



; start timeout movie
; uses: TMP
;  - used stack depth (without call to this function): 0
TIMEOUT_START:
        ldi     TMP,0x01                        ; movie starts with next tick
        mov     TIMEOUT_TICKS,TMP               ;  - not set to 0x00 because this would result
                                                ;    in 0xFF after decrementation in timeout task
        clr     TIMEOUT_FRAME                   ; (re)start timeout movie
        ret



; timeout task
;  - uses: Y, Z, CNT, TMP
;  - used stack depth (without call to this function): 1
TIMEOUT_TASK:
; wait for next 100ms to pass
        in      TMP,TIFR                        ; return if 100ms flag (timer 1 compare match) is notset
        sbrs    TMP,OCF1A
        ret
        cli                                     ; turn off interrupts
        in      TMP,OCF1A                       ; clear 100ms flag (timer 1 compare match)
        sbr     TMP,1<<OCF1A
        out     TIFR,TMP
        sei                                     ; turn on interrupts
; count down 100ms intervals and check if timeout elapsed
        dec     TIMEOUT_TICKS
        brne    TIMEOUT_TASK_END
; get address of current frame in memory
        mov     YL,TIMEOUT_FRAME                ; get offset of current frame into Y
        lsl     YL                              ;  - multiply frame index with 18*3+2=56=7*8
        lsl     YL
        lsl     YL
        sub     YL,TIMEOUT_FRAME
        clr     YH                              ;    - multiplied with 7, now multiply with 8
        lsl     YL
        rol     YH
        lsl     YL
        rol     YH
        lsl     YL
        rol     YH
        ldi     ZH,high(2*MOVIE_DATA)           ; load base address of movie data ("2*" to convert word address to byte address)
        ldi     ZL,low(2*MOVIE_DATA)
        add     ZL,YL                           ; add offset
        adc     ZH,YH
; output picture
        rcall   OUT_PIC
; next frame
        inc     TIMEOUT_FRAME
; get new timeout (time before to play next frame)
        lpm     TIMEOUT_TICKS,Z+                ; get number of ticks to wait
; check for end of movie
        lpm     TMP,Z+                          ; read end flag
        cpi     TMP,0                           ; 0 indicates end of movie
        brne    TIMEOUT_TASK_REWIND_END
        clr     TIMEOUT_FRAME                   ; rewind movie
TIMEOUT_TASK_REWIND_END:
TIMEOUT_TASK_END:
        ret
