; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; transmission code
; version 0.2.1 date 2006-06-17
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; serial tranmission
; uses: Z, SNES_FLAG, TRANS_STEP, CNT, TMP
;  - used stack depth (without call to this function): 0
TRANS:
        sbrc    SNES_FLAG,SNES_F_DATA           ; check for new data to transmit
        rjmp    TRANS_DO
        clr     TRANS_STEP                      ; no new data -> restart with byte 0 if new data arrives
        ret
TRANS_DO:
        sbis    UCSRA,UDRE                      ; exit if no space in USART transmit buffer
        ret
        clr     REG0                            ; get SNES mask (1st key) into REG0 (mask1 := 1 << (step / 2) & 3)
        inc     REG0
        sbrc    TRANS_STEP,2
        lsl     REG0
        sbrc    TRANS_STEP,2
        lsl     REG0
        sbrc    TRANS_STEP,1
        lsl     REG0
        mov     REG1,REG0                       ; get SNES mask (2nd key) into REG1 (mask2 := mask1 << 4)
        swap    REG1
        sbrc    TRANS_STEP,0                    ; transmitting high or low byte? (highbyte is transmitted first)
        rjmp    TRANS_LOW
        ldi     ZH,high(SNES_BUF+6)             ; get address of "key 12/13" in SNES key buffer
        ldi     ZL,low(SNES_BUF+6)
        clr     REG2                            ; get data to transmit (SNES buttons 7..11)
        ldi     CNT,3
TRANS_HIGH_LOOP:
        lsl     REG2
        ld      TMP,-Z                          ;  - load 2 keys
        and     TMP,REG1                        ;  - 2nd key
        brne    TRANS_HIGH_BIT0                 ;  - SNES key bits are inverted
        inc     REG2
TRANS_HIGH_BIT0:
        lsl     REG2
        ld      TMP,Z                           ;  - load 2 keys again
        and     TMP,REG0                        ;  - 1st key
        brne    TRANS_HIGH_BIT00                ;  - SNES key bits are inverted
        inc     REG2
TRANS_HIGH_BIT00:
        dec     CNT
        brne    TRANS_HIGH_LOOP
        cpi     TRANS_STEP,0                    ; set sync bit (D7) if first byte of first SNES pad
        brne    TRANS_FINISH
        ldi     TMP,0x80
        or      REG2,TMP
        rjmp    TRANS_FINISH
TRANS_LOW:
        ldi     ZH,high(SNES_BUF+3)             ; get address of key 6/7 in SNES key buffer
        ldi     ZL,low(SNES_BUF+3)
        clr     REG2                            ; get data to transmit (SNES buttons 0..6)
        ldi     CNT,3
TRANS_LOW_LOOP:
        lsl     REG2
        ld      TMP,-Z                          ;  - load 2 keys
        and     TMP,REG1                        ;  - 2nd key
        brne    TRANS_LOW_BIT0                  ;  - SNES key bits are inverted
        inc     REG2
TRANS_LOW_BIT0:
        lsl     REG2
        ld      TMP,Z                           ;  - load 2 keys again
        and     TMP,REG0                        ;  - 1st key
        brne    TRANS_LOW_BIT00                 ;  - SNES key bits are inverted
        inc     REG2
TRANS_LOW_BIT00:
        dec     CNT
        brne    TRANS_LOW_LOOP
TRANS_FINISH:
        out     UDR,REG2                        ; transmit data
        inc     TRANS_STEP                      ; next step
        cpi     TRANS_STEP,8                    ; finished?
        brlo    TRANS_END
        cbr     SNES_FLAG,1<<SNES_F_DATA        ; SNES data is sent
        clr     TRANS_STEP                      ; restart with first byte next time
TRANS_END:
        ret
