; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; register definitions
; version 0.2.1 date 2006-06-17
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; register X is reserved for usage in interrupt !!!



; general purpose registers for interrupt
.def    I_TMP           =       r16
.def    I_CNT           =       r17
.def    I_REG0          =       r0
.def    I_REG1          =       r1
.def    I_REG2          =       r2

; registers for bits of 9th shift register chain in interrupt
.def    I_COL17         =       r3
.def    I_COL18         =       r4

; status register backup in interrupt
.def    I_SREG          =       r5



; general purpose registers
.def    TMP             =       r18
.def    CNT             =       r19
.def    REG0            =       r6
.def    REG1            =       r7
.def    REG2            =       r8



; current greyscale step
;  - counts 49 steps (0..48) to output 8 quadratic greyscale values
;  - for i = 0..6:
;     - in step i^2: output black/white picture, turn on pixel with value > i
;  - in step 42: read SNES keys (1st half)
;  - in step 46: read SNES keys (2nd half)
.def    STEP            =       r20

; addresses of output buffers to use (see also OUT_BUF_*)
;  - flag to indicate assignment of output buffers
;    - 0         output buffer 1 for output, output buffer 2 for receiving
;    - 1 (!= 0): output buffer 2 for output, output buffer 1 for receiving
.def    OUT_PTR_FLAG    =       r21
;  - pointer to end of output buffer (used for output to LEDs)
.def    OUT_PTR_OUT_E_L =       r9
.def    OUT_PTR_OUT_E_H =       r10
;  - pointer to receive buffer (used for receiving new values)
.def    OUT_PTR_RECV_L  =       r11
.def    OUT_PTR_RECV_H  =       r12

; flag to indicate that new SNES key data to be sent to PC is available
;  - bit SNES_F_DATA
;    - 0: no new SNES key data available,
;         interrupt may fetch new data and set this flag
;    - 1: new SNES key data is available
;         main loop may sent data and reset flag
;  - bit SNES_F_2ND
;    - set by interrupt to indicate that 2nd half of work must be done
.def    SNES_FLAG       =       r22
.equ    SNES_F_DATA     =       0
.equ    SNES_F_2ND      =       1

; receive step
;  - counts bytes received so far
.def    RECV_STEP       =       r23

; receive coordinates
;  - x coordinate of next pixel
.def    RECV_X          =       r13
;  - y coordinate of next pixel
.def    RECV_Y          =       r14

; transmit step
;  - counts bytes to send
.def    TRANS_STEP      =       r24

; timeout tick (100ms) counter
;  - counts down ticks left to wait before playing next frame
.def    TIMEOUT_TICKS   =       r25
; timeout frame number
;  - index of next frame to play
.def    TIMEOUT_FRAME   =       r15
